/**
 * Program: ClientUtils.java
 * Comments: This class collects several static utility methods that client 
 * programs will typically use.  Note this isn't a WebFlow module.
 */

package WebFlow;

import java.io.*;
import java.net.*;
import java.util.*;
import org.omg.CORBA.*;
import javax.swing.*;
import WebFlow.ContextManager.*;
import WebFlow.hashtable.*;

public class ClientUtils {
    
    /**
     * Return the IOR from the specified file.
     */
    public static String getIORFromURL(String urlString){
	String masterServerIOR="";
	String thisLine=null;
	try {
	    URL u = new URL(urlString);
	    try {
		DataInputStream theHTML = new DataInputStream(u.openStream());
		try {
		    while ((thisLine = theHTML.readLine()) != null) {
			masterServerIOR += thisLine;
		    } 
		} 
		catch (Exception e) {
		    System.out.println("getIORfromURL:"+e);
		}
	    } 
	    catch (Exception e) {
		System.out.println("getIORfromURL:"+e);
        }
	    
	} 
	catch (MalformedURLException e) {
	    System.out.println("getIORfromURL:"+urlString 
			       + " is not a parseable URL");
	    System.out.println("getIORfromURL:"+e);
	}
	return masterServerIOR;
    }
    
    /**
     * Initialize the orb.  Set the properties to override the ORB 
     * in the sun release.  
     */
    public static ORB initializeORB(String IorURL,ORB orb) {
	Properties props = System.getProperties();
	props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
	props.put("org.omg.CORBA.ORBSingletonClass", 
		  "com.ooc.CORBA.ORBSingleton");
	System.setProperties(props);
	String[] args = new String[1];
	args[0]=IorURL;
	orb = ORB.init(args, props);
	return orb;
    }

    /**
     * Initialize the orb for an applet.  Because of security restrictions,
     * system properties cannot be modified by an applet.
     */
    public static ORB initAppletORB(JApplet applet) {
	ORB orb = ORB.init(applet,new java.util.Properties());
	System.out.println("Orb is initialized");
	return orb;
    }

    /**
     * Create the webflow master server.
     */
    public static WebFlowContext getMasterServer(String masterIOR,ORB orb){
	WebFlowContext ws = null;
	System.out.println("masterIOR is "+masterIOR);
	try {
	    org.omg.CORBA.Object o = orb.string_to_object(masterIOR);
	    ws = WebFlowContextHelper.narrow(o);
	}
	catch (Exception e) {
	    System.out.println("getMasterServer: "+e);
	    e.printStackTrace();
	}
	return ws;
    }

    /**
     * Create a module object.  The user will have to narrow this.  
     */

    public static org.omg.CORBA.Object 
	makeModule(WebFlowContext master,String moduleName) {
	org.omg.CORBA.Object obj=null;
	if(master==null) {
	    System.err.println("Server context is null");
	}
	else {
	    obj=master.addNewModule(moduleName);
	}
	return obj;
    }

    /**
     * Restore all of the child contexts of the specified parent.
     */
    public static void restoreChildren(WebFlowContext context, 
				       ContextManager manager,
				       SaveContext save) {
	System.out.println("starting restoration ...");
	ContextManager cm=manager;
	SaveContext sc=save;
        if(cm==null) System.out.println("cm is null"); 
        if(context==null) System.out.println("context is null"); 
	String children=cm.getContextData(context,"Children");
	System.err.println("children: "+children); 
	String pDir=cm.getContextData(context,"Directory");
	System.err.println("directory: "+pDir); 
	if(children.equals("null")) { 
	    // no more children: job done
	    System.out.println("no more children: job done");
	    return;
	}
	else {
	    //Children are in format:  name1+name2+name3
	    StringTokenizer st = new StringTokenizer(children,"+");
	    while(st.hasMoreTokens()) {
		System.out.println("has more tokens");
		String Child = st.nextToken();
		System.err.println("restore context "+Child);
		// create an empty context and put ContextData module to it
		WebFlowContext cc = restoreContext(context,Child);
		// a context name may have a suffix: .PC, .SC, or .AC. strip it
		String cDir = Child;
		int idx = Child.lastIndexOf(".");
		if(idx>-1) {
		    cDir = Child.substring(0,idx); 
             }  
		cDir = pDir+File.separator + cDir;
		System.out.println("cDir="+cDir);  
		sc.readContextData(cc,cDir);
             restoreChildren(cc,cm,sc);
	    }
	}
    }
    
    public static WebFlowContext restoreContext(WebFlowContext p, String name){
	try {
	    WebFlowContext c=
		WebFlowContextHelper.narrow(p.addNewContext(name));
	    if(c==null) System.out.println("c is null");
	    ContextData cdm=
		ContextDataHelper.narrow(c.addNewModule("ContextData"));
	    if(cdm==null) System.out.println("--->cd is null");
	    return c;
	} catch (Exception e) {
	    System.out.println(e +"while createGatewayContext");
       return null;
	}  
    }
}

