/**
 * Program Name: createPBS.java
 * Comments: This class is a child of the createScript abstract parent.  
 * Creates a PBS script.
 * 
 * Note that the ContextManager object cm and the string scriptString are
 * protected data inherited from the parent.
 *
 * I am assuming the following naming convention for the context data:
 *
 *   shell: the unix shell to use.
 *   queue: the particular queue (default, challenge, and so on).
 *   Name: the name to give the job.
 *   NNodes: the number of nodes to request.
 *   walltime: the maximum time per node.
 * 
 * Note that all of these variables are not required.
 */

package extensions.generic;
import WebFlow.*;
import WebFlow.ContextManager.*;
import java.io.*;
import java.util.*;

public class createPBS extends createScript {
    Hashtable ht;

    public createPBS(ContextManager cm,String userName){
	super(cm,userName);
    }

    /**
     * This method concatenates the PBS-specific header directives
     * to the string scriptString, inherited from the parent.
     * It probably wouldn't hurt at some point to associate the 
     * param names with their flags (shell and -S, for example).
     */
    public void createHeader() {
	scriptString+="#! /bin/csh \n";
	checkHash();
    }
    
    public void createFlagTable() {
	ht=new Hashtable();
	ht.put("shell","#PBS -S ");
	ht.put("queue","#PBS -q ");
	ht.put("Name","#PBS -N ");
	ht.put("NNodes","#PBS -l nodes=");
	ht.put("walltime","#PBS -l walltime=");
	
	return;
    }

    private void checkHash() {
	String name,val;
	Enumeration en=ht.keys();
	while(en.hasMoreElements()){
	    name=(String)en.nextElement();
	    if(name.equals("Name")) {
		scriptString+=(String)ht.get(name)
		    +cm.getCurrentProblemProperty(name)+"\n";
	    }
	    else if(!(cm.getCurrentSessionProperty(name)==null)){
		scriptString+=(String)ht.get(name)
		    +cm.getCurrentSessionProperty(name)+"\n";
	    }    
	}
    }
}





